/* Copyright (C) 2005, 2006 Hartmut Holzgraefe, MySQL AB

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA */

#include <stdio.h>
#include <stdlib.h>
#include <getopt.h>

#include <mysql.h>
#include <NdbApi.hpp>

#include "config.h"

extern "C" int load_defaults(const char *conf_file, const char **groups,int *argc, char ***argv);

#define BARK(obj, msg) do { fprintf(stderr, "%s: %s (%d) in %s:%d\n", msg, obj->getNdbError().message, obj->getNdbError().code, __FILE__, __LINE__); } while (0);
#define BAILOUT(obj, msg) do { BARK(obj, msg) exit(3); } while (0);

Ndb_cluster_connection *example_init(int *pargc, char ***pargv)
{
	char *connectstr = "localhost";
	int no_retries   = 4;
	int retry_delay  = 5; 
	int verbose      = 1;

	Ndb_cluster_connection *conn;

	// initialize MySQL and Ndb client libraries
	if (ndb_init()) {
		exit(EXIT_FAILURE);
	}

	// import configuration 
	const char * mycnf_sections[] = {"mysql_cluster", NULL};
	load_defaults("my", mycnf_sections, pargc, pargv);

	// parse command line arguments
	struct option long_options[] = {
		{"connect_string", 1, 0, 'c'},
		{"try_reconnect" , 1, 0, 't'},
		{0, 0, 0, 0}
	};
	char *short_options = "c:";
	int opt, option_index;

	opterr = 0;
	do {
		opt = getopt_long(*pargc, *pargv, short_options, long_options, &option_index);
		switch (opt) {
		case 'c':
			connectstr = optarg;
			break;
		case 't':
			no_retries = atoi(optarg);
		default:
			break;
		}
	} while ( opt != -1);

	// prepare connection to cluster
	printf("trying to connect to '%s'\n", connectstr);
	conn = new Ndb_cluster_connection(connectstr);

	// initiate connection
	if (conn->connect(no_retries, retry_delay, verbose))
	{
		fprintf(stderr, "Unable to connect to cluster.");
		exit(EXIT_FAILURE);
	}
	
	// wait for the storage nodes (ndbd's)
	if (conn->wait_until_ready(30,0) < 0)
	{
		fprintf(stderr, "Cluster was not ready within 30 secs.\n");
		exit(EXIT_FAILURE);
	}

	return conn; 
}

void example_end(Ndb_cluster_connection *conn)
{
	// tear down connection
	delete conn;

	// shut down MySQL and Ndb client libraries
	ndb_end(0);
}

const char *durabilityName(NdbDictionary::Event::EventDurability d)
{
  switch (d)
  {
  case NdbDictionary::Event::ED_UNDEFINED: return "undefined";
#if 0 // see bug #33552
  case NdbDictionary::Event::ED_SESSION:   return "session";
  case NdbDictionary::Event::ED_TEMPORARY: return "temporary";
#endif
  case NdbDictionary::Event::ED_PERMANENT: return "permanent";
  default: return "???";
  }
}

const char *reportName(NdbDictionary::Event::EventReport r)
{
  switch (r)
  {
  case NdbDictionary::Event::ER_UPDATED:   return "updated";
  case NdbDictionary::Event::ER_ALL:       return "all";
  case NdbDictionary::Event::ER_SUBSCRIBE: return "subscribed";
  default: return "???";
  }
}

const char *tableEventName(NdbDictionary::Event::TableEvent e)
{
  switch (e)
  {
  case NdbDictionary::Event::TE_INSERT:          return "insert";
  case NdbDictionary::Event::TE_DELETE:          return "delete";
  case NdbDictionary::Event::TE_UPDATE:          return "update";
  case NdbDictionary::Event::TE_SCAN:            return "scan";
  case NdbDictionary::Event::TE_DROP:            return "drop";
  case NdbDictionary::Event::TE_ALTER:           return "alter";
  case NdbDictionary::Event::TE_CREATE:          return "create";
  case NdbDictionary::Event::TE_GCP_COMPLETE:    return "gcp_complete";
  case NdbDictionary::Event::TE_CLUSTER_FAILURE: return "cluster_failure";
  case NdbDictionary::Event::TE_STOP:            return "stop";
  case NdbDictionary::Event::TE_NODE_FAILURE:    return "node_failure";
  case NdbDictionary::Event::TE_SUBSCRIBE:       return "subscribe";
  case NdbDictionary::Event::TE_UNSUBSCRIBE:     return "unsubscribe";
  default: return "???";
  }
}

int main(int argc, char **argv)
{
	Ndb_cluster_connection *conn;
	char constr[1024];

	conn = example_init(&argc, &argv);

	Ndb *ndb = new Ndb(conn, "test");
	if (!ndb) {
	  fprintf(stderr, "Can't create NDB object\n");
	  exit(3);
	}
	if (ndb->init()) {
	  fprintf(stderr, "Can't create NDB object\n");
	  exit(3);
	}

	NdbDictionary::Dictionary *dict = ndb->getDictionary();

	while (true) {
	  NdbDictionary::Dictionary::List events;
	  if (dict->listEvents(events)) {
		BAILOUT(dict, "can't get event list");
	  }
	  
	  for (int i = 0; i < events.count; i++) {
		NdbDictionary::Dictionary::List::Element *e;
		
		printf("%2d: %s\n", i+1, events.elements[i].name);
	  }

	  printf("\nEvent number for details, [ENTER] to reload, Q to quit\n");

	  char buf[80], *line;

	  line = fgets(buf, sizeof(buf), stdin);

	  if (!strcasecmp(line, "Q\n")) break;

	  int item = atoi(line);

	  if (item > 0 && item <= events.count) {
		const NdbDictionary::Event *e;

		printf("\nGetting details for event '%s'\n", events.elements[item - 1].name);

		e = dict->getEvent(events.elements[item - 1].name);

		if (!e) { 
		  BARK(dict, "can't get event");
		} else {
		  printf("getName(): '%s'\n", e->getName());
		  printf("getTableName(): '%s'\n", e->getTableName());
		  
		  printf("getTableEvent():");
#define CHECK_TABLE_EVENT(x) if (e->getTableEvent(NdbDictionary::Event::x)) printf(" %s", tableEventName(NdbDictionary::Event::x)); 
		  CHECK_TABLE_EVENT(TE_INSERT);
		  CHECK_TABLE_EVENT(TE_DELETE);
		  CHECK_TABLE_EVENT(TE_UPDATE);
		  CHECK_TABLE_EVENT(TE_SCAN);
		  CHECK_TABLE_EVENT(TE_DROP);
		  CHECK_TABLE_EVENT(TE_ALTER);
		  CHECK_TABLE_EVENT(TE_CREATE);
		  CHECK_TABLE_EVENT(TE_GCP_COMPLETE);
		  CHECK_TABLE_EVENT(TE_CLUSTER_FAILURE);
		  CHECK_TABLE_EVENT(TE_STOP);
		  CHECK_TABLE_EVENT(TE_NODE_FAILURE);
		  CHECK_TABLE_EVENT(TE_SUBSCRIBE);
		  CHECK_TABLE_EVENT(TE_UNSUBSCRIBE);
		  printf("\n");
		  
		  
		  printf("getDurability(): %s\n", durabilityName(e->getDurability()));
		  printf("getReport(): %s\n", reportName(e->getReport()));
		  printf("getNoOfEventColumns(): %d\n", e->getNoOfEventColumns());
		  
		  printf("getEventColumn(): ");
		  for (unsigned i = 0; i < e->getNoOfEventColumns(); i++) {
			const NdbDictionary::Column *col = e->getEventColumn(i);
			
			printf("%s ", col->getName());
		  }
		  printf("\n");
		  
		}

		printf("\n\n");
	  }
	}

	example_end(conn);

	return EXIT_SUCCESS;
}
